#
# Created by Yi MIN<yi@zen.ly>
# Copyright © 2017 Zenly. All rights reserved.
#
Dir[File.dirname(__FILE__) + '/wda_lib/*.rb'].each {|file| require file }
require 'json'
require 'httparty'
require 'selenium-webdriver'
Dir[File.dirname(__FILE__) + '/selenium_patch/*.rb'].each {|file| require_relative file }

class WDA

  include HTTParty
  headers 'Content-Type' => 'application/json'
  # extend Forwardable
  # def_delegators :@driver, :find_element, :find_elements, :action, :touch, :element_scroll
  # def_delegator :@driver, :get, :get_url

  include ::WDA::Alert
  include ::WDA::Custome
  include ::WDA::Debug
  include ::WDA::FindElement
  include ::WDA::Orientation
  include ::WDA::Screenshot
  include ::WDA::Session
  include ::WDA::TouchID
  include ::WDA::Type
  include ::WDA::Error

  Point     = Struct.new(:x, :y)
  Dimension = Struct.new(:width, :height)

  attr_accessor :session_id, :http, :bundle_id, :caps, :device, :driver, :timeout
  attr_reader :base_url, :server_host, :server_port, :url, :window_w, :window_h, :win_x, :win_y

  # Get base_url from XCTRunner logs: ServerURLHere->http://x.x.x.x:8100<-ServerURLHere
  # Default base_url 'http://localhost:8100' for running XCTRunner on simulator
  BASE_URL = 'http://localhost:8100'

  # Create a new client
  # 
  # ```ruby
  # require 'rubygems'
  # require 'wda_lib'
  # 
  # Start wda client
  # opts = { caps: { bundleId: @bundle_id }, device_url: 'http://x.x.x.x:8100' }
  # WDA.new(opts).start_session
  def initialize(opt)
    # fail 'opts should be a hash' unless opts.is_a? Hash
    # fail 'bundleId should be provided' if opts[:bundle_id].nil?
    if opt.is_a? String
      url = opt
      opts = {:device_url => url}
    else
      opts = opt
    end
    url = opts[:device_url] || BASE_URL
    parsed_url = URI.parse(url)
    @base_url = parsed_url.scheme + '://' + parsed_url.host + ':' + parsed_url.port.to_s
    @url = @base_url
    @server_host = parsed_url.host
    @server_port = parsed_url.port
    @default_timeout = opts.fetch :default_timeout, 10
    @timeout = @default_timeout
    @listener = opts.fetch :listener, nil
    status
    window_size
    capabilities(opts)
  end

  # Build desired_capabilities with options
  # @param opts [Hash]
  # @return attribute caps [Hash]
  def capabilities(opts = {}) 
    @bundle_id = opts[:bundle_id] if !opts[:bundle_id].nil?
    selenium_capabilities = Selenium::WebDriver::Remote::Capabilities.new( 
                              platform_name: @platform_name, 
                              platform_version: @platform_version,
                              session_id: @session_id,
                              session_valid: @session_valid,
                              bundleId: @bundle_id,
                              javascript_enabled: true,
                              takes_screenshot: true,
                              css_selectors_enabled: true
                            )
    @caps = { desired_capabilities: selenium_capabilities }
  end

  # HTTP methods for dealing with all supported motheds in FB WDA
  # I don't like to write such pieces of code, it removes all query params and headers,
  # But as FB WDA is doing like this, this can simplify codes
  def get(path, timeout = 60)
    if URI.parse(path).absolute?
      WDA.get(path, timeout: timeout)
    else
      WDA.get(@url + path, timeout: timeout)
    end
  end

  def post(path, body = nil, timeout = 60)
    if URI.parse(path).absolute?
      WDA.post(path, body: body.to_json, timeout: timeout)
    else
      WDA.post(@url + path, body: body.to_json, timeout: timeout)
    end
  end

  def delete(path)
    WDA.delete(path)
  end

  def set_wait(timeout = nil)
    if timeout.nil?
      @timeout = @default_timeout
    elsif timeout < 1
      @timeout = 1
    else
      @timeout = timeout.to_i
    end
    return @timeout
  end
end