#
# Created by Yi MIN<yi@zen.ly>
# Copyright © 2017 Zenly. All rights reserved.
#
class WDA
  class Element

    def initialize(client, id)
      @client = client
      @id = id
    end

    # Remove useless attributes, return object only with element
    def inspect
      format '#<%s:0x%x id=%s>', self.class, hash * 2, @id.inspect
    end

    def ref
      @id
    end

    def eid
      @id['ELEMENT']
    end

    # Check if element is enabled?
    # @return is isEnabled? [Boolean]
    def enabled?
      client.get('/element/' + eid + '/enabled')['value']
    end

    # Get wdRect by id
    # @return is wdRect 
    def rect
      client.get('/element/' + eid + '/rect')['value']
    end

    # Get attribute value
    # @param name [String]
    # @return is attributeValue [String]
    def attribute
      client.get '/element/' + eid + '/attribute/name'
    end

    # Get text from an element(StaticText or Button)
    # @return is text [String]
    def text
      client.get('/element/' + eid + '/text')['value']
    end

    # Check if element is displayed?
    # @return is isVisible? [Boolean]
    def displayed?
      client.get('/element/' + eid + '/displayed')['value']
    end
    
    # Check if element is accessible?
    # @return is accessible? [Boolean]
    def accessible?
      client.get('/wda/element/' + eid + '/accessible')['value']
    end
    
    # Check if element is accessibilityContainer?
    # @return is accessibilityContainer? [Boolean]
    def accessible_container?
      client.get('/wda/element/' + eid + '/accessibilityContainer')['value']
    end

    # Get type from an element, ex:  type => "Icon"(XCUIElementTypeIcon)
    # @return is type [String]
    def type
      client.get('/element/' + eid + '/name')['value']
    end

    def name
      type
    end

    # Set value to an element
    # @param value [String]
    # @return element uuid [String]
    def send_keys(value)
      client.post '/element/' + eid + '/value', { value: value.chars }
    end

    # Tap on an element with its id
    # @return element uuid [String]
    def click
      client.post '/element/' + eid + '/click'
    end

    # Clearing text of an given element
    # @return element uuid [String]
    def clear
      client.post '/element/' + eid + '/clear'
    end

    # Element swipe
    # @param direction [String] up, down, left, right
    # @return element uuid [String]
    def elswipe(direction)
      client.post('/wda/element/' + eid + '/swipe', { direction: direction })
    end

    # Element pinch
    # @param scale [Double], velocity [Double]
    # @return element uuid [String]
    def elswipe(scale, velocity)
      client.post('/wda/element/' + eid + '/pinch', { scale: scale, velocity: velocity })
    end

    # Double tap on an element with its id
    # @return element uuid [String]
    def double_tap
      client.post '/wda/element/' + eid + '/doubleTap'
    end

    # Two finger tap on an element with its id
    # @return element uuid [String]
    def two_finger_tap
      client.post '/wda/element/' + eid + '/twoFingerTap'
    end

    # Touch and hold on for a while finger tap on an element with its id
    # @param duration [Double]
    # @return element uuid [String]
    def touch_hold(duration)
      client.post '/wda/element/' + eid + '/touchAndHold', { duration: duration }
    end

    # Scroll on an element with its id
    # @param direction [String] up, down, left, right
    # @return [Hash]
    def scroll(direction = nil)
      client.post '/wda/element/' + eid + '/scroll', { direction: direction }
    end

    # Drag on an element with its id and position
    # @param fromX [Double], toX [Double], fromY [Double], toY [Double], duration [Double]
    def drag_element_from_to(fromX, toX, fromY, toY, duration = 0)
      client.post '/wda/element/' + eid + '/dragfromtoforduration', { fromX: fromX, toX: toX, fromY: fromY, toY: toY, duration: duration }
    end

    # Drag from a position to another position
    # @param fromX [Double], toX [Double], fromY [Double], toY [Double], duration [Double]
    def drag_from_to(fromX, toX, fromY, toY, duration = 0)
      client.post '/wda/dragfromtoforduration', { fromX: fromX, toX: toX, fromY: fromY, toY: toY, duration: duration }
    end

    # Get element size
    # @return [Struct] [:width, :height]
    def size
      r = rect['value']
      Dimension.new(r['width'], r['height'])
    end

    # Get element size
    # @return [Struct] [:x, :y]
    def location
      r = rect
      Point.new(r['x'], r['y'])
    end

    alias_method :[], :attribute

    private

    attr_reader :client

  end

  # Swipe on an element with its id and position
  # @param fromX [Integer], toX [Integer], fromY [Integer], toY [Integer]
  def swipe(fromX, toX, fromY, toY)
    post '/wda/dragfromtoforduration', { fromX: fromX, toX: toX, fromY: fromY, toY: toY, duration: 0 }
  end

  # Tap on a given positon
  # @param x [Integer], y [Integer]
  def tap_on(x, y)
    post '/wda/tap/nil', { x: x, y: y}
  end

  # Double tap on a given positon
  # @param x [Integer], y [Integer]
  def double_tap(x, y)
    post '/wda/doubleTap', { x: x, y: y }
  end

  # Touch and hold on for a while
  # @param x [Double], y [Double], duration [Double]
  # @return element uuid [String]
  def touch_hold(x, y, duration)
    client.post '/wda/touchAndHold', { x: x, y: y, duration: duration }
  end

  # Type text to textfield
  # @param value [String] 
  def keys(value)
    post '/wda/keys', { value: value.chars }
  end

  # @return width, height
  def window_size
    win_size = get('/window/size')['value']
    @win_x = win_size['width']
    @win_y = win_size['height']
    Dimension.new(win_size['width'], win_size['height'])
  end

end